package com.yyrh.ui;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Paint;
import android.os.CountDownTimer;
import android.text.method.HideReturnsTransformationMethod;
import android.text.method.PasswordTransformationMethod;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.yyrh.constant.SDKSettings;
import com.yyrh.sdk.YyrhSdkManager;
import com.yyrh.utils.ResourceUtil;
import com.yyrh.utils.TosUtil;
import com.yyrh.utils.Utils;
import com.yyrh.networking.NetRequestApi;
import com.yyrh.sdk.SDKEntry;
import com.yyrh.sdk.callback.LoginCallback;
import com.yyrh.ui.userlist.DatabaseHelper;
import com.yyrh.ui.userlist.User;
import com.yyrh.ui.userlist.UserListAdapter;

import java.util.ArrayList;

public class LoginLayout implements View.OnClickListener {



    private Context context;

    private Button loginButton,registButton,forgetPwdBtn;

    private EditText accountEdit, pwdEdit;

    private ImageView showListView,showPwd;

    private boolean isShowList = false;

    private ConstantDialog constantDialog;

    private boolean isShowPwd = false;


    DatabaseHelper dbHelper;
    public TextView tv;
    private UserListAdapter adapter;
    private ListView listView;


    public LoginLayout(Context context,ConstantDialog constantDialog){

        this.context = context;
        this.constantDialog = constantDialog;
        if (SDKSettings.isLandscape){
            constantDialog.setContentView(ResourceUtil.getLayoutId(context, "xy_login_dialog"));
        }else{
            constantDialog.setContentView(ResourceUtil.getLayoutId(context, "xy_login_dialog_protrait"));
        }

        loginButton 		=  constantDialog.findViewById(ResourceUtil.getId(context, "xy_enter_btn"));
        registButton 		=  constantDialog.findViewById(ResourceUtil.getId(context, "xy_reg_btn"));
        listView 			=  constantDialog.findViewById(ResourceUtil.getId(context, "account_listview"));
        accountEdit 		=  constantDialog.findViewById(ResourceUtil.getId(context, "xy_user_edit"));
        pwdEdit 			=  constantDialog.findViewById(ResourceUtil.getId(context, "xy_pwd_edit"));
        showListView 		= constantDialog.findViewById(ResourceUtil.getId(context, "down_btn"));
        showPwd = constantDialog.findViewById(ResourceUtil.getId(context, "down_eye_btn"));
        tv 					= constantDialog.findViewById(ResourceUtil.getId(context, "list_text"));
        forgetPwdBtn = constantDialog.findViewById(ResourceUtil.getId(context, "xy_forget_pwd_btn"));

        showListView.setOnClickListener(this);
        loginButton.setOnClickListener(this);
        registButton.setOnClickListener(this);
        showPwd.setOnClickListener(this);
        forgetPwdBtn.setOnClickListener(this);

        SharedPreferences sp = context.getSharedPreferences("xiyuan", Activity.MODE_PRIVATE);
        accountEdit.setText(sp.getString("username", ""));
        pwdEdit.setText(sp.getString("password", ""));
        pwdEdit.setTransformationMethod(PasswordTransformationMethod.getInstance());


        adapter = new UserListAdapter(this,context,getData());
        listView.setAdapter(adapter);
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                accountEdit.setText(adapter.list.get(position).uname);
                pwdEdit.setText(adapter.list.get(position).pwd);
                hideListView();
            }
        });
        if (SDKSettings.isHideFunctionsOfPhone){
            forgetPwdBtn.setVisibility(View.GONE);
        }
    }



    @Override
    public void onClick(View v) {

        int id = v.getId();
        if (id == ResourceUtil.getId(context, "xy_enter_btn")) { 			/*	进入游戏		*/

            startLogin(accountEdit.getText().toString(), pwdEdit.getText().toString());

        } else if (id == ResourceUtil.getId(context, "xy_reg_btn")) {		/*	1秒注册		*/
            new RegisterLayout(context,constantDialog);

        }else if (id == ResourceUtil.getId(context, "xy_forget_pwd_btn")) {		/*	1秒注册		*/
//            new TosUtil(context,"blue", context.getString(ResourceUtil.getStringId(context, "xy_function_unopen"))).show();

             new ForgetPwdLayout(context,constantDialog);

        } else if (id == ResourceUtil.getId(context, "down_eye_btn")) {		/*	1秒注册		*/
            if (!isShowPwd) {
                isShowPwd = true;
                pwdEdit.setTransformationMethod(HideReturnsTransformationMethod.getInstance());
                showPwd.setImageResource(ResourceUtil.getMipmapId(context, "xy_eye_icon_select"));
            } else {
                isShowPwd = false;
                pwdEdit.setTransformationMethod(PasswordTransformationMethod.getInstance());
                showPwd.setImageResource(ResourceUtil.getMipmapId(context, "xy_eye_icon_normal"));
            }

        }else if (id == ResourceUtil.getId(context, "down_btn")){

            /*	显示缓存账号	*/
            if (!isShowList) {
                showListView.setImageResource(ResourceUtil.getMipmapId(context, "xy_arrow_select"));
                showListView();
            } else {

                showListView.setImageResource(ResourceUtil.getMipmapId(context, "xy_arrow_normal"));
                hideListView();
            }
        }

    }

    private void showListView() {
        if (adapter.list.size() == 0) {
            tv.setVisibility(View.VISIBLE);
        } else {
            listView.setVisibility(View.VISIBLE);
            tv.setVisibility(View.GONE);

        }
        isShowList = true;


    }

    private void hideListView() {
        listView.setVisibility(View.GONE);
        tv.setVisibility(View.GONE);
        isShowList = false;

        showListView.setImageResource(ResourceUtil.getMipmapId(context, "xy_arrow_normal"));
    }


    private void startLogin(final String account, final String pwd) {
        if (Utils.judgeStrNull(account)){
            new TosUtil(context,"blue", context.getString(ResourceUtil.getStringId(context, "xy_band_need_account"))).show();
           // ToastUtil.show(context, context.getString(ResourceUtil.getStringId(context, "xy_band_need_account")));
            return;
        }
        if (Utils.judgeStrNull(pwd)){
            new TosUtil(context,"blue", context.getString(ResourceUtil.getStringId(context, "xy_band_need_pwd"))).show();
          //  ToastUtil.show(context, context.getString(ResourceUtil.getStringId(context, "xy_band_need_pwd")));
            return;
        }

        if(account.length()<6)
        {
            new TosUtil(context,"blue", context.getString(ResourceUtil.getStringId(context, "xy_account_limit_6"))).show();
         //   ToastUtil.show(context, context.getString(ResourceUtil.getStringId(context, "xy_account_limit_6")));
            return;
        }
        if(pwd.length()<6)
        {
            new TosUtil(context,"blue", context.getString(ResourceUtil.getStringId(context, "xy_pwd_limit_6"))).show();
           // ToastUtil.show(context, context.getString(ResourceUtil.getStringId(context, "xy_pwd_limit_6")));
            return;
        }
        registerInterval();
        NetRequestApi.loginWithYy(true,(Activity) context, account, pwd, new LoginCallback() {
            @Override
            public void onLoginSuccess(String token) {

                constantDialog.dismiss();


                YyrhSdkManager.defaultManager(context).saveUserLoginInfo(context,account,pwd,token);

            }

            @Override
            public void onLoginFailed(String msg) {
                new TosUtil(context,"blue", msg).show();

                SDKEntry.getSdkInstance().loginCall.onLoginFailed(msg);
            }

            @Override
            public void onLoginError(String msg) {
                new TosUtil(context,"blue", msg).show();
            }
        });

    }

    private ArrayList<User> getData() {
        ArrayList<User> userList = new ArrayList<User>();
        dbHelper = new DatabaseHelper(context);
        SQLiteDatabase db = null;
        db = dbHelper.getReadableDatabase();
        String qurey_sql = "select name,pwd from user order by time desc";
        Cursor cursor = db.rawQuery(qurey_sql, null);
        while (cursor.moveToNext()) {
            User user = new User();
            user.uname = cursor.getString(0);
            user.pwd = cursor.getString(1);
            userList.add(user);
        }
        cursor.close();
        db.close();
        return userList;
    }

    private void registerInterval() {


        loginButton.setEnabled(false);
        loginButton.setText("3s");
        loginButton.setBackgroundResource(ResourceUtil.getDrawableId(context,"xy_radius_deepgray_bg"));
        TimeCount time = new TimeCount(3000, 1000);
        time.start();
    }


    class TimeCount extends CountDownTimer {

        public TimeCount(long millisInFuture, long countDownInterval) {
            super(millisInFuture, countDownInterval);
            // TODO Auto-generated constructor stub
        }

        @Override
        public void onTick(long millisUntilFinished) {
            // TODO Auto-generated method stub
            loginButton.setText(millisUntilFinished / 1000 + "s");

        }

        @Override
        public void onFinish() {
            // TODO Auto-generated method stub
            loginButton.setEnabled(true);
            loginButton.setBackgroundResource(ResourceUtil.getDrawableId(context,"xy_radius_main_color_bg"));
            loginButton.setText(ResourceUtil.getStringId(context, "xy_enter_game"));
        }

    }
}
